from base import APC2016Base
from jsk import APC2016jsk
from mit_benchmark import APC2016mit_benchmark
from rbo import APC2016rbo


class APC2016V2(APC2016Base):

    def __init__(self, root, train, transform):
        self.datasets = [
            APC2016jsk(root, train, transform),
            APC2016rbo(root, train, transform),
            APC2016mit_benchmark(root, train, transform),
        ]

    def __len__(self):
        return sum(len(d) for d in self.datasets)

    @property
    def train(self):
        train = self.datasets[0].train
        assert all(d.train == train for d in self.datasets)
        return train

    @train.setter
    def train(self, value):
        for d in self.datasets:
            d.train = value

    def __getitem__(self, index):
        skipped = 0
        for dataset in self.datasets:
            current_index = index - skipped
            if current_index < len(dataset):
                return dataset[current_index]
            skipped += len(dataset)



import torch
import torchvision


def VGG16(pretrained=False):
    model = torchvision.models.vgg16(pretrained=False)
    if not pretrained:
        return model
    model_url = 'https://s3-us-west-2.amazonaws.com/jcjohns-models/vgg16-00b39a1b.pth'  # NOQA
    state_dict = torch.utils.model_zoo.load_url(model_url)
    # patch state_dict
    state_dict['classifier.0.weight'] = state_dict.pop('classifier.1.weight')
    state_dict['classifier.0.bias'] = state_dict.pop('classifier.1.bias')
    state_dict['classifier.3.weight'] = state_dict.pop('classifier.4.weight')
    state_dict['classifier.3.bias'] = state_dict.pop('classifier.4.bias')
    model.load_state_dict(state_dict)
    return model



import glob
import os
import os.path as osp
import re

import numpy as np
import scipy.misc
from sklearn.model_selection import train_test_split

from base import APC2016Base


class APC2016rbo(APC2016Base):

    def __init__(self, root, train=True, transform=False):
        self.train = train
        self._transform = transform
        self.dataset_dir = osp.join(root, 'apc2016/APC2016rbo')
        data_ids = self._get_ids()
        ids_train, ids_val = train_test_split(
            data_ids, test_size=0.25, random_state=1234)
        self._ids = {'train': ids_train, 'val': ids_val}

    def __len__(self):
        split = 'train' if self.train else 'val'
        return len(self._ids[split])

    def _get_ids(self):
        ids = []
        for img_file in os.listdir(self.dataset_dir):
            if not re.match(r'^.*_[0-9]*_bin_[a-l].jpg$', img_file):
                continue
            data_id = osp.splitext(img_file)[0]
            ids.append(data_id)
        return ids

    def _load_from_id(self, data_id):
        img_file = osp.join(self.dataset_dir, data_id + '.jpg')
        img = scipy.misc.imread(img_file)
        # generate label from mask files
        lbl = np.zeros(img.shape[:2], dtype=np.int32)
        # shelf bin mask file
        shelf_bin_mask_file = osp.join(self.dataset_dir, data_id + '.pbm')
        shelf_bin_mask = scipy.misc.imread(shelf_bin_mask_file, mode='L')
        lbl[shelf_bin_mask < 127] = -1
        # object mask files
        mask_glob = osp.join(self.dataset_dir, data_id + '_*.pbm')
        for mask_file in glob.glob(mask_glob):
            mask_id = osp.splitext(osp.basename(mask_file))[0]
            mask = scipy.misc.imread(mask_file, mode='L')
            lbl_name = mask_id[len(data_id + '_'):]
            lbl_id = np.where(self.class_names == lbl_name)[0]
            lbl[mask > 127] = lbl_id
        return img, lbl

    def __getitem__(self, index):
        split = 'train' if self.train else 'val'
        data_id = self._ids[split][index]
        img, lbl = self._load_from_id(data_id)
        if self._transform:
            return self.transform(img, lbl)
        else:
            return img, lbl



import numpy as np
import torch
import torch.nn as nn


# https://github.com/shelhamer/fcn.berkeleyvision.org/blob/master/surgery.py
def get_upsample_filter(size):
    """Make a 2D bilinear kernel suitable for upsampling"""
    factor = (size + 1) // 2
    if size % 2 == 1:
        center = factor - 1
    else:
        center = factor - 0.5
    og = np.ogrid[:size, :size]
    filter = (1 - abs(og[0] - center) / factor) * \
             (1 - abs(og[1] - center) / factor)
    return torch.from_numpy(filter).float()


class FCN32s(nn.Module):

    def __init__(self, n_class=21, nodeconv=False):
        super(FCN32s, self).__init__()
        self.nodeconv = nodeconv
        self.features = nn.Sequential(
            # conv1
            nn.Conv2d(3, 64, 3, padding=100),
            nn.ReLU(inplace=True),
            nn.Conv2d(64, 64, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, stride=2, ceil_mode=True),  # 1/2

            # conv2
            nn.Conv2d(64, 128, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.Conv2d(128, 128, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, stride=2, ceil_mode=True),  # 1/4

            # conv3
            nn.Conv2d(128, 256, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.Conv2d(256, 256, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.Conv2d(256, 256, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, stride=2, ceil_mode=True),  # 1/8

            # conv4
            nn.Conv2d(256, 512, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.Conv2d(512, 512, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.Conv2d(512, 512, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, stride=2, ceil_mode=True),  # 1/16

            # conv5
            nn.Conv2d(512, 512, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.Conv2d(512, 512, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.Conv2d(512, 512, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, stride=2, ceil_mode=True),  # 1/32
        )
        self.classifier = nn.Sequential(
            # fc6
            nn.Conv2d(512, 4096, 7),
            nn.ReLU(inplace=True),
            nn.Dropout2d(),

            # fc7
            nn.Conv2d(4096, 4096, 1),
            nn.ReLU(inplace=True),
            nn.Dropout2d(),

            # score_fr
            nn.Conv2d(4096, n_class, 1),
        )
        if self.nodeconv:
            self.upscore = nn.UpsamplingBilinear2d(scale_factor=32)
            self.upscore.scale_factor = None  # XXX: shoud be set in forward
        else:
            self.upscore = nn.ConvTranspose2d(n_class, n_class, 64, stride=32,
                                              bias=False)

    def forward(self, x):
        h = self.features(x)

        h = self.classifier(h)

        if self.nodeconv:
            from chainer.utils import conv
            in_h, in_w = h.size()[2:4]
            out_h = conv.get_deconv_outsize(in_h, k=64, s=32, p=0)
            out_w = conv.get_deconv_outsize(in_w, k=64, s=32, p=0)
            self.upscore.size = out_h, out_w
            h = self.upscore(h)
        else:
            h = self.upscore(h)
        h = h[:, :, 19:19+x.size()[2], 19:19+x.size()[3]].contiguous()

        return h

    def copy_params_from_vgg16(self, vgg16, copy_fc8=True, init_upscore=True):
        for l1, l2 in zip(vgg16.features, self.features):
            if (isinstance(l1, nn.Conv2d) and
                    isinstance(l2, nn.Conv2d)):
                assert l1.weight.size() == l2.weight.size()
                assert l1.bias.size() == l2.bias.size()
                l2.weight.data = l1.weight.data
                l2.bias.data = l1.bias.data
        for i in [0, 3]:
            l1 = vgg16.classifier[i]
            l2 = self.classifier[i]
            l2.weight.data = l1.weight.data.view(l2.weight.size())
            l2.bias.data = l1.bias.data.view(l2.bias.size())
        n_class = self.classifier[6].weight.size()[0]
        if copy_fc8:
            l1 = vgg16.classifier[6]
            l2 = self.classifier[6]
            l2.weight.data = l1.weight.data[:n_class, :].view(l2.weight.size())
            l2.bias.data = l1.bias.data[:n_class]
        if init_upscore:
            # initialize upscore layer
            c1, c2, h, w = self.upscore.weight.data.size()
            assert c1 == c2 == n_class
            assert h == w
            weight = get_upsample_filter(h)
            self.upscore.weight.data = \
                weight.view(1, 1, h, w).repeat(c1, c2, 1, 1)



import numpy as np

import torch
from torch.utils import data


class APC2016Base(data.Dataset):

    class_names = np.array([
        'background',
        'barkely_hide_bones',
        'cherokee_easy_tee_shirt',
        'clorox_utility_brush',
        'cloud_b_plush_bear',
        'command_hooks',
        'cool_shot_glue_sticks',
        'crayola_24_ct',
        'creativity_chenille_stems',
        'dasani_water_bottle',
        'dove_beauty_bar',
        'dr_browns_bottle_brush',
        'easter_turtle_sippy_cup',
        'elmers_washable_no_run_school_glue',
        'expo_dry_erase_board_eraser',
        'fiskars_scissors_red',
        'fitness_gear_3lb_dumbbell',
        'folgers_classic_roast_coffee',
        'hanes_tube_socks',
        'i_am_a_bunny_book',
        'jane_eyre_dvd',
        'kleenex_paper_towels',
        'kleenex_tissue_box',
        'kyjen_squeakin_eggs_plush_puppies',
        'laugh_out_loud_joke_book',
        'oral_b_toothbrush_green',
        'oral_b_toothbrush_red',
        'peva_shower_curtain_liner',
        'platinum_pets_dog_bowl',
        'rawlings_baseball',
        'rolodex_jumbo_pencil_cup',
        'safety_first_outlet_plugs',
        'scotch_bubble_mailer',
        'scotch_duct_tape',
        'soft_white_lightbulb',
        'staples_index_cards',
        'ticonderoga_12_pencils',
        'up_glucose_bottle',
        'womens_knit_gloves',
        'woods_extension_cord',
    ])
    mean_bgr = np.array([104.00698793, 116.66876762, 122.67891434])

    def transform(self, img, lbl):
        img = img[:, :, ::-1]  # RGB -> BGR
        img = img.astype(np.float64)
        img -= self.mean_bgr
        img = img.transpose(2, 0, 1)
        img = torch.from_numpy(img).float()
        lbl = torch.from_numpy(lbl).long()
        return img, lbl

    def untransform(self, img, lbl):
        img = img.numpy()
        img = img.transpose(1, 2, 0)
        img += self.mean_bgr
        img = img.astype(np.uint8)
        img = img[:, :, ::-1]
        lbl = lbl.numpy()
        return img, lbl



#!/usr/bin/env python

from __future__ import print_function

import sys

from setuptools import find_packages
from setuptools import setup


__version__ = '1.5.0'


try:
    import torch  # NOQA
except ImportError:
    print('Please install PyTorch. (See http://pytorch.org)', file=sys.stderr)
    sys.exit(1)


setup(
    name='torchfcn',
    version=__version__,
    packages=find_packages(),
    install_requires=[r.strip() for r in open('requirements.txt')],
    description='PyTorch Implementation of Fully Convolutional Networks.',
    package_data={'torchfcn': ['ext/*']},
    include_package_data=True,
    author='Kentaro Wada',
    author_email='www.kentaro.wada@gmail.com',
    license='MIT',
    url='https://github.com/wkentaro/torchfcn',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



import itertools
import os
import os.path as osp

import numpy as np
import scipy.misc
from sklearn.model_selection import train_test_split

from base import APC2016Base


def ids_from_scene_dir(scene_dir, empty_scene_dir):
    for i_frame in itertools.count():
        empty_file = osp.join(
            empty_scene_dir, 'frame-{:06}.color.png'.format(i_frame))
        rgb_file = osp.join(
            scene_dir, 'frame-{:06}.color.png'.format(i_frame))
        segm_file = osp.join(
            scene_dir, 'segm/frame-{:06}.segm.png'.format(i_frame))
        if not (osp.exists(rgb_file) and osp.exists(segm_file)):
            break
        data_id = (empty_file, rgb_file, segm_file)
        yield data_id


def bin_id_from_scene_dir(scene_dir):
    caminfo = open(osp.join(scene_dir, 'cam.info.txt')).read()
    loc = caminfo.splitlines()[0].split(': ')[-1]
    if loc == 'shelf':
        bin_id = caminfo.splitlines()[1][-1]
    else:
        bin_id = 'tote'
    return bin_id


class APC2016mit_benchmark(APC2016Base):

    def __init__(self, root, train=True, transform=False):
        self.train = train
        self._transform = transform
        self.dataset_dir = osp.join(root, 'apc2016/benchmark')
        data_ids = self._get_ids()
        ids_train, ids_val = train_test_split(
            data_ids, test_size=0.25, random_state=1234)
        self._ids = {'train': ids_train, 'val': ids_val}

    def __len__(self):
        split = 'train' if self.train else 'val'
        return len(self._ids[split])

    def _get_ids_from_loc_dir(self, env, loc_dir):
        assert env in ('office', 'warehouse')
        loc = osp.basename(loc_dir)
        data_ids = []
        for scene_dir in os.listdir(loc_dir):
            scene_dir = osp.join(loc_dir, scene_dir)
            bin_id = bin_id_from_scene_dir(scene_dir)
            empty_dir = osp.join(
                self.dataset_dir, env, 'empty', loc, 'scene-{}'.format(bin_id))
            data_ids += list(ids_from_scene_dir(scene_dir, empty_dir))
        return data_ids

    def _get_ids(self):
        data_ids = []
        # office
        contain_dir = osp.join(self.dataset_dir, 'office/test')
        for loc in ['shelf', 'tote']:
            loc_dir = osp.join(contain_dir, loc)
            data_ids += self._get_ids_from_loc_dir('office', loc_dir)
        # warehouse
        contain_dir = osp.join(self.dataset_dir, 'warehouse')
        for sub in ['practice', 'competition']:
            sub_contain_dir = osp.join(contain_dir, sub)
            for loc in ['shelf', 'tote']:
                loc_dir = osp.join(sub_contain_dir, loc)
                data_ids += self._get_ids_from_loc_dir('warehouse', loc_dir)
        return data_ids

    def _load_from_id(self, data_id):
        empty_file, rgb_file, segm_file = data_id
        img = scipy.misc.imread(rgb_file, mode='RGB')
        img_empty = scipy.misc.imread(empty_file, mode='RGB')
        # Label value is multiplied by 9:
        #   ex) 0: 0/6=0 (background), 54: 54/6=9 (dasani_bottle_water)
        lbl = scipy.misc.imread(segm_file, mode='L') / 6
        lbl = lbl.astype(np.int32)
        img_empty[lbl > 0] = img[lbl > 0]
        return img_empty, lbl

    def __getitem__(self, index):
        split = 'train' if self.train else 'val'
        data_id = self._ids[split][index]
        img, lbl = self._load_from_id(data_id)
        if self._transform:
            return self.transform(img, lbl)
        else:
            return img, lbl
